function adjrand=adjrand(Palg,Qalg,P,Q)

% Computes the adjusted Rand index to assess the quality of a clustering.
% Perfectly random clustering returns the minimum score of 0, perfect
% clustering returns the maximum score of 1.
%
%INPUTS
% Palg, Qalg = the labeling as predicted by a clustering algorithm
% P,Q = the true labeling
%
%OUTPUTS
% adjrand = the adjusted Rand index
%
%Author: Tijl De Bie, february 2003.
%Modified for two-mode partitioning by Joost van Rosmalen
Palgtemp=Palg*(1:1:size(Palg,2))';
Qalgtemp=Qalg*(1:1:size(Qalg,2))';
[n,K]=size(Palg);
[m,L]=size(Qalg);
v=zeros(n*m,1);
for y=1:n
    for z=1:m
        v((y-1)*m+z)=(Palgtemp(y)-1)*L+Qalgtemp(z);
    end
end
Porgtemp=P*(1:1:size(P,2))';
Qorgtemp=Q*(1:1:size(Q,2))';
u=zeros(n*m,1);
for y=1:n
    for z=1:m
        u((y-1)*m+z)=(Porgtemp(y)-1)*L+Qorgtemp(z);
    end
end
u=u*(1:1:size(u,2))';
v=v*(1:1:size(v,2))';
n=length(u);
ku=max(u);
kv=max(v);
m=zeros(ku,kv);
for i=1:n
    m(u(i),v(i))=m(u(i),v(i))+1;
end
mu=sum(m,2);
mv=sum(m,1);
a=0;
for i=1:ku
    for j=1:kv
        if m(i,j)>1
            a=a+nchoosek(m(i,j),2);
        end
    end
end
b1=0;
b2=0;
for i=1:ku
    if mu(i)>1
        b1=b1+nchoosek(mu(i),2);
    end
end
for i=1:kv
    if mv(i)>1
        b2=b2+nchoosek(mv(i),2);
    end
end
c=nchoosek(n,2);
adjrand=(a-b1*b2/c)/(0.5*(b1+b2)-b1*b2/c);