function results=simstudy
%This function performs the simulation study of Van Rosmalen et al.,
%Journal of Classification (forthcoming)
%Warning: this function may require more than a day of computation time.

%choose number of repetitions
nrepetitions=50;
%initialize results matrix
%dim 1: algorithm
%dim 2: repetition number
%dim 3: result type,
%1= vaf,
%2= fraction of rows classified correctly
%3= fraction of columns classified correctly, 4= computation time
%dim 4-7: factors 1-4
results=zeros(6,nrepetitions,3,3,3,3,3);
%choose parameters of simulation
errorlevel=[.5 1 2];
fraconecluster=[0,.1,.6];
for i=1:nrepetitions
    for factor4=1:3
        for factor1=1:3
            if factor1==1
                n=60;
                m=60;
            elseif factor1==2
                n=150;
                m=30;
            elseif factor1==3
                n=120;
                m=120;
            end
            for factor2=1:3
                perturbation=errorlevel(factor2);
                for factor3=1:3
                    if factor3==1
                        K=3;
                        L=3;
                    elseif factor3==2
                        K=5;
                        L=5;
                    elseif factor3==3
                        K=7;
                        L=7;
                    end                  
                    % vectors with results of simulation study
                    % row 1: AE
                    % row 2: two-mode k-means
                    % row 3: simulated annealing
                    % row 4: tabu search
                    % row 5: fuzzy algorithm
                    % row 6: original partition
                  
                    %construct random original P and Q
                    [P,Q]=randompartition(m,n,K,L,fraconecluster(factor4));
                    %generate V
                    V=zeros(K,L);
                    normquantiles=norminv(1/(K*L+1):1/(K*L+1):1-1/(K*L+1));
                    V(:)=normquantiles(randperm(K*L));
                    %    generate X
                    X=P*V*Q'+perturbation*(randn(n,m));
                
                    %  calculate variance of data for fast calculation of VAFs
                    variance=sum(sum((X-sum(sum(X))/(n*m)).^2));
                    %construct random starting points for P and Q
                    [Ptemp,Qtemp]=randompartition(m,n,K,L,0);
                    %improve these using kmeans algorithm
                    [vafvalue,Ptemp,Qtemp]=kmeans(Ptemp,Qtemp,X,m,n,K,L,variance);

                    %perform the five algorithms on P,Q and V and store the
                    %VAFs and computation times in results
                    nmultistart=30;
                    tic;
                    [vafvalue,Palg,Qalg]=multiAE(X,m,n,K,L,variance,nmultistart);
                    results(1,i,1,factor1,factor2,factor3,factor4)=vafvalue;
                    results(1,i,2,factor1,factor2,factor3,factor4)=toc;
                    results(1,i,3,factor1,factor2,factor3,factor4)=adjrand(Palg,Qalg,P,Q);

                    nmultistart=500;
                    tic;
                    [vafvalue,Palg,Qalg]=multikmeans(X,m,n,K,L,variance,nmultistart);
                    results(2,i,1,factor1,factor2,factor3,factor4)=vafvalue;
                    results(2,i,2,factor1,factor2,factor3,factor4)=toc;
                    results(2,i,3,factor1,factor2,factor3,factor4)=adjrand(Palg,Qalg,P,Q);

                    temperature=1;
                    gamma=0.95;
                    maxiter=round(.5*(n*(K-1)+m*(L-1)));
                    maxnnotfrozen=10;
                    tic;
                    [vafvalue,Palg,Qalg]=simanneal(Ptemp,Qtemp,X,m,n,K,L,variance,temperature,gamma,maxiter,maxnnotfrozen);
                    results(3,i,1,factor1,factor2,factor3,factor4)=vafvalue;
                    results(3,i,2,factor1,factor2,factor3,factor4)=toc;
                    results(3,i,3,factor1,factor2,factor3,factor4)=adjrand(Palg,Qalg,P,Q);

                    tabulength=round(1*(sqrt(n*(K-1)+m*(L-1))));
                    maxiter=round(3*(sqrt(n*(K-1)+m*(L-1))));
                    tic;
                    [vafvalue,Palg,Qalg]=tabusearch(Ptemp,Qtemp,X,m,n,K,L,variance,tabulength,maxiter);
                    results(4,i,1,factor1,factor2,factor3,factor4)=vafvalue;
                    results(4,i,2,factor1,factor2,factor3,factor4)=toc;
                    results(4,i,3,factor1,factor2,factor3,factor4)=adjrand(Palg,Qalg,P,Q);

                    s=1.05;
                    gamma=0.85;
                    s_min=1.001;
                    tic;
                    [vafvalue,Palg,Qalg]=multifuzzy(X,m,n,K,L,variance,s,gamma,s_min,20);
                    results(5,i,1,factor1,factor2,factor3,factor4)=vafvalue;
                    results(5,i,2,factor1,factor2,factor3,factor4)=toc;
                    results(5,i,3,factor1,factor2,factor3,factor4)=adjrand(Palg,Qalg,P,Q);

                    %calculate vaf of original partition
                    results(6,i,1,factor1,factor2,factor3,factor4)=1-sum(sum((X-P*(P'*X*Q./(sum(P)'*sum(Q)))*Q').^2))/variance;

                end
            end
        end
    end
end
